from urllib.parse import urlparse, quote
from .download_file import download_file
from .ssl_adapter import SingletonSession
import requests
import os
import time

def get_dataset_resource(dataset_id,allowed_exts=['csv', 'xlsx', 'xls'], output_dir=f"opendata/org_resources",ext_dir=None, headers = None, verbose = None):
    """For each dataset, download the available resources that meet the extensions criteria

    Args:
        dataset_id (str): The dataset ID to download resources from
        allowed_exts (list, optional): The list of allowed file extensions to try to download. Defaults to ['csv', 'xlsx', 'xls'].
        output_dir (str, optional): The directory to save the downloaded files. Defaults to f"opendata/org_resources".
        headers (list, optional): The list of headers to use for the request. Defaults to None.
        ext_dir (bool, optional): The directory to save the downloaded files. Defaults to None.
        verbose (bool, optional): Whether to print verbose output. Defaults to None.

    Returns:
        None: No value returned
    """
    #Assign the default headers to get the correesponding resources
    if headers is None:
        headers = {
        'Accept': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Referer': 'https://open.data.gov.sa/',
        'Accept-Language': 'en-US,en;q=0.9',
    }
    #Create the session item for the requests instance
    session = SingletonSession.get_instance()
    dataset_params = {
            'version': '-1',
            'dataset': dataset_id
        }
    dataset_response = session.get('https://open.data.gov.sa/data/api/datasets/resources', params=dataset_params, headers=headers)

    # Check if the response contains valid JSON
    try:
        dataset_data = dataset_response.json()
    except requests.exceptions.JSONDecodeError:
        print(f"Failed to decode JSON for dataset {dataset_id}")
        return None
    parent_dir = output_dir
    for resource in dataset_data['resources']:
            if verbose:print(f"Attempting to download resource: {resource['downloadUrl']}")
            download_url = resource['downloadUrl']
            parsed_url = urlparse(download_url)
            # Extract the file extension
            file_name = os.path.basename(parsed_url.path)
            file_extension = os.path.splitext(file_name)[1].lstrip('.')

            # Skip the file if its extension is not in the allowed list
            if file_extension not in allowed_exts:
                if verbose:print(f"Skipping file with extension {file_extension}: {download_url}")
                continue
            
            #URLs to try
            safe_url = parsed_url._replace(path=quote(parsed_url.path, safe='/')).geturl()
            lr_url = f"https://open.data.gov.sa/data/api/v1/datasets/{dataset_data['datasetId']}/resources/{resource['id']}/download"

            # Construct the output file path
            output_dir = parent_dir if ext_dir is None else os.path.join(parent_dir, file_extension)
            resource_file_path = os.path.join(output_dir, file_name)
            
            # Ensure the output directory exists
            os.makedirs(output_dir, exist_ok=True)

            # Check if the file already exists and its size
            if os.path.exists(resource_file_path) and os.path.getsize(resource_file_path) > 250:
                if(verbose):print(f"Skipping existing file: {resource_file_path}")
                continue

            # Check if the file already exists, its size, and its age
            if os.path.exists(resource_file_path):
                file_age = time.time() - os.path.getmtime(resource_file_path)
                if os.path.getsize(resource_file_path) > 250 and file_age <= 7 * 24 * 60 * 60:
                    if(verbose):print(f"Skipping existing file: {resource_file_path}")
                    continue
                elif file_age > 7 * 24 * 60 * 60:
                    if(verbose):print(f"Deleting old file: {resource_file_path}")
                    os.remove(resource_file_path)

            # Add headers to mimic a browser request
            download_headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': f'https://open.data.gov.sa/',
                'Accept-Language': 'en-US,en;q=0.9'
            }
            
            if(verbose):
                print(f"OG URL: {download_url}")
                print(f"SA URL: {safe_url}")
            
            # Attempt to download using the safe URL
            file_size = download_file(session,safe_url, download_headers, resource_file_path)
            
            # If the file is less than 250 bytes, attempt to download using the original URL
            if file_size == 0:
                if(verbose):print(f"File {file_name} is less than 250 bytes, retrying with original URL")
                file_size = download_file(session,lr_url, download_headers, resource_file_path)
            
            if file_size > 250:
                if(verbose):print(f"Downloaded and saved file: {resource_file_path}")
            else:
                if(verbose):print(f"Failed to download a valid file for: {file_name}")




# open_ksa/__init__.py
from setuptools_scm import get_version

try:
    # Attempt to get the version from the SCM metadata
    __version__ = get_version()
except LookupError:
    # Fallback to a default version if SCM metadata is not available
    __version__ = "0.0.0"
    
from urllib.parse import urlparse, quote
from .download_file import download_file
from .get_dataset_resource import get_dataset_resource
from .get_dataset_resources import get_dataset_resources
from .get_org_resources import get_org_resources
from .ssl_adapter import SSLAdapter, SingletonSession
from .organizations import organizations


#examples/scripts/3_get_dataset_resources.py

from open_ksa import get_dataset_resources,get_org_resources

def test_get_dataset_resources():
    # Define the dataset ID to test
    org_id = 'a9e617ff-d918-4f4d-8be1-c42b733b1143' #King Saud University
    resources = get_org_resources(org_id=org_id)
    #Here, we grab all of the different dataset_ids
    dataset_ids = resources['dataset_ids']
    #Here, we grab the organization ID as well. But we can use the same organization ID from the ks value
    # we named it ks for 'King Saud University'
    organization_id = resources['organization_id']

    get_dataset_resources(dataset_ids=dataset_ids, 
                         output_dir=f"opendata/{resources['organization_name'].strip().replace(' ', '_').lower()}",
                         allowed_exts=['csv'],
                         verbose=False
                         )

if __name__ == "__main__":
    test_get_dataset_resources()


# -*- coding: utf-8 -*-
#
# Downloading all of the datasets of a single organization ID, if applicable from the opendata portal from KSA
# An organization's full list of resources is retrieved
# Then, we go through all resources to collect the downloadURL links
# Once all downloadURL's are collected, an array is made and a loop goes through each download URL
# Downloads all of the files to a folder called 'opendata/{organizationid}'
# Not, some datasets are still broken, but some decent coverage


#Here you can import all of the corresponding functions from the workbook
import open_ksa as ok

#An example on how to use the search function
orgs = ok.organizations(search="saudi")

#Note: Orgs as a value has the export of the full JSON taken from the API

#Here, we grab the first value which is the value of the organization ID from the API
#Depending on the parameters, we can specify the return of the response
ks = orgs['content'][0]['publisherID']
# We have now gotten the publisher ID programmatically. If you change the ID to a string of your choosing or decide to 
#change the search, you can change the orgs['content'][0]['publisherID'] to match your search and the index 0 to N to 
#to match the organization you want
resources = ok.get_org_resources(org_id=ks)
#Here, we grab all of the different dataset_ids
dataset_ids = resources['dataset_ids']
#Here, we grab the organization ID as well. But we can use the same organization ID from the ks value
# we named it ks for 'King Saud University'
organization_id = resources['organization_id']

# Create a directory named after the organization ID
# Get all of the data resources for the organization
ok.get_dataset_resources(dataset_ids=dataset_ids, 
                        #You can update the dataset resource location to change the output directory
                        #Note: you may have to make the directory
                         output_dir=f"opendata/{resources['organization_name'].strip().replace(' ', '_').lower()}"
                         )



import ssl
import sys
from urllib3 import poolmanager
from requests.adapters import HTTPAdapter
import requests

class SSLAdapter(HTTPAdapter):
    """
    An HTTPS Transport Adapter that enables SSL connections
    """
    def init_poolmanager(self, *args, **kwargs):
        context = ssl.create_default_context()
        if sys.version_info >= (3, 12):
            context.options |= ssl.OP_LEGACY_SERVER_CONNECT
        kwargs['ssl_context'] = context
        return super(SSLAdapter, self).init_poolmanager(*args, **kwargs)

class SingletonSession:
    _instance = None

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = requests.Session()
            cls._instance.mount('https://', SSLAdapter())
        return cls._instance


import requests
import json
import csv
import os

def organizations(headers=None, size=400, page=0, sort='datasetsCount,DESC',search=None, target=None):
    """Fetch organizations from the API and save to a file if target is specified.

    Args:
        header (dict, optional): The header to use for the request of the API call. Defaults to None.
        size (int, optional): The size of the response. Defaults to 400.
        page (int, optional): The page number. Defaults to 0.
        sort (str, optional): The sort order. Defaults to 'datasetsCount,DESC'.
        target (str, optional): The file path to save the data. Defaults to None.

    Returns:
        dict: The response from the API call for the organization.
    """
    if headers is None:
        
        headers = {
        'Accept': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Referer': 'https://open.data.gov.sa/',
        'Accept-Language': 'en-US,en;q=0.9',
        'Host':'open.data.gov.sa',
        'Upgrade-Insecure-Requests':'1'
        }


    params = {'size': size, 'page': page, 'sort': sort}
    if search is not None:
        params['term'] = search

 

    response = requests.get('https://open.data.gov.sa/api/organizations', params=params,headers=headers)
    
    if response.status_code == 200:
        try:
            # Try to parse JSON response
            data = response.json()
            
            if target is not None:
                # Check if target is a complete filepath
                if not os.path.isabs(target):
                    target = os.path.join(os.getcwd(), target)
                
                _, file_extension = os.path.splitext(target)
                if file_extension.lower() == '.json':
                    # Write data to JSON file
                    with open(target, 'w') as jsonfile:
                        json.dump(data, jsonfile, indent=4)
                    return data
                elif file_extension.lower() == '.csv':
                    # Write data to CSV file
                    with open(target, 'w', newline='') as csvfile:
                        writer = csv.writer(csvfile)
                        # Write headers
                        writer.writerow(data['content'][0].keys())
                        # Write data rows
                        for item in data['content']:
                            writer.writerow(item.values())
            else:
                # Extract the first 10 items
                items = data['content'][:10]

                # Define the headers
                headers = ["publisherID", "slug","Datasets"]

                # Calculate the maximum width for each column
                max_widths = [len(header) for header in headers]
                for item in items:
                    max_widths[0] = max(max_widths[0], len(item['publisherID']))
                    max_widths[1] = max(max_widths[1], len(item['slug']))
                    max_widths[2] = max(max_widths[2], len(str(item['numberOfDatasets'])))

                # Create a format string for each row
                row_format = "| {:<" + str(max_widths[0]) + "} | {:<" + str(max_widths[1]) + "} | {:<" + str(max_widths[2]) + "} |"

                # Print the header
                print(row_format.format(*headers))
                print("-" * (sum(max_widths) + 7))

                # Print each row
                for item in items:
                    print(row_format.format(item['publisherID'], item['slug'],item['numberOfDatasets']))

                return data
        except requests.exceptions.JSONDecodeError:
            print("Failed to decode JSON. Here is the raw response:")
            print(response.text)
            # Print the raw response if JSON decoding fails



# examples/scripts/1_organizations.py
from open_ksa import organizations

def main():
    # Call the organizations function and print the result
    orgs = organizations()
    print("\n\nList of Organizations:\n")
    for org in orgs['content'][:10]:
        print(org['name'])

if __name__ == "__main__":
    main()


import requests
from .ssl_adapter import SingletonSession
from urllib.parse import urlparse, quote

def get_org_resources(org_url = 'https://open.data.gov.sa/data/api/organizations',org_id = 'd69f01cd-ef1b-47e4-ad23-8e58a8d5a468'):
    session = SingletonSession.get_instance()
    
    headers = {
        'Accept': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Referer': 'https://open.data.gov.sa/',
        'Accept-Language': 'en-US,en;q=0.9',
    }

    #Parameters set for the organization - 
    #Here, we have defined this for the saudi port authority
    # To select a different organization, please visit the following website and copy the organization ID:  https://open.data.gov.sa/en/publishers
    # By finding the organization, visiting their page and copying the organization ID from the URL
    #
    params = {
        'version': '-1',
        'organization': org_id
    }

    # Make the request using the custom session
    response = session.get(org_url, params=params, headers=headers)
    
    try:
            data = response.json()
    except requests.exceptions.JSONDecodeError:
            print(f"Failed to decode JSON for dataset {org_id}")
    
    #print(data['nameEn'])
    return {'organization_name':data['nameEn'], 
            'organization_id':data['id'], 
            'dataset_ids':[dataset['id'] for dataset in data['datasets']]
            }



# tests/test_get_dataset_resource.py
import unittest
from unittest.mock import patch, MagicMock
from open_ksa.get_dataset_resource import get_dataset_resource  # Adjust the import based on your package structure
import os

class TestGetDatasetResource(unittest.TestCase):

    @patch('open_ksa.get_dataset_resource.SingletonSession.get_instance')
    @patch('open_ksa.get_dataset_resource.download_file')
    @patch('os.makedirs')
    @patch('os.path.exists')
    @patch('os.path.getsize')
    @patch('os.remove')
    def test_get_dataset_resource_successful_download(
        self, mock_remove, mock_getsize, mock_exists, mock_makedirs, mock_download_file, mock_session_get_instance
    ):
        # Setup mocks
        mock_session = MagicMock()
        mock_session_get_instance.return_value = mock_session

        # Mock dataset response
        mock_response = MagicMock()
        mock_response.json.return_value = {
            'resources': [
                {
                    'downloadUrl': 'https://example.com/file.csv',
                    'id': 'resource_id_1',
                }
            ],
            'datasetId': 'dataset_id_1'
        }
        mock_session.get.return_value = mock_response

        # Mock os.path.exists and os.path.getsize
        mock_exists.return_value = False
        mock_getsize.return_value = 0

        # Mock download_file to return a file size
        mock_download_file.return_value = 1024  # Assume 1KB file size

        # Call the function
        get_dataset_resource('dataset_id_1', verbose=True)

        # Assertions for the initial API call to get dataset resources
        mock_session.get.assert_called_with(
            'https://open.data.gov.sa/data/api/datasets/resources',
            params={'version': '-1', 'dataset': 'dataset_id_1'},
            headers={
                'Accept': 'application/json',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': 'https://open.data.gov.sa/',
                'Accept-Language': 'en-US,en;q=0.9',
                'Host': 'open.data.gov.sa',
                'Upgrade-Insecure-Requests': '1'
            }
        )

        # Assertions for the download process
        mock_makedirs.assert_called_once_with('opendata/org_resources', exist_ok=True)
        mock_download_file.assert_called_once_with(
            mock_session, 'https://open.data.gov.sa/data/api/v1/datasets/dataset_id_1/resources/resource_id_1/download', 
            {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': 'https://open.data.gov.sa/en/datasets/view/dataset_id_1/resources',
                'Accept': 'application/json, text/javascript, */*; q=0.01', 
                'Accept-Language': 'en-US,en;q=0.9', 
                'Host': 'open.data.gov.sa', 
                'Upgrade-Insecure-Requests': '1', 
                'X-Requested-With': 'XMLHttpRequest', 
                'Connection': 'keep-alive'
            }, 
            'opendata/org_resources/file.csv'
        )

if __name__ == "__main__":
    unittest.main()



import os
import requests
import concurrent.futures
import time

from .download_file import download_file
from .get_dataset_resource import get_dataset_resource
from urllib.parse import urlparse, quote
from .ssl_adapter import SingletonSession


def get_dataset_resources(dataset_ids,allowed_exts=['csv', 'xlsx', 'xls'],output_dir=f"opendata/org_resources",verbose = None, ext_dir=None, max_workers=None):
    """Download the resources for each dataset in the list of dataset IDs

    Args:
        dataset_ids (list): The list of dataset IDs to download resources from
        allowed_exts (list, optional): The list of allowed file extensions to try to download. Defaults to ['csv', 'xlsx', 'xls'].
        output_dir (str, optional): The directory to save the downloaded files. Defaults to f"opendata/org_resources".
        verbose (bool, optional): Whether to print verbose output. Defaults to False.
        ext_dir (bool, optional): The directory to save the downloaded files. Defaults to None.
        max_workers (int, optional): The maximum number of workers to use for the concurrent download. Defaults to None.

    Returns:
        None: No value returned. Files downloaded to specified directory in `output_dir`
    """
    session = SingletonSession.get_instance()
    headers = {
        'Accept': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Referer': 'https://open.data.gov.sa/',
        'Accept-Language': 'en-US,en;q=0.9',
    }

    if not os.path.exists(output_dir):
        os.makedirs(output_dir, exist_ok=True)
        # Function to download a single dataset resource
    def download_resource(dataset_id):
        get_dataset_resource(dataset_id=dataset_id, allowed_exts=allowed_exts, output_dir=output_dir, headers=headers, ext_dir=ext_dir, verbose=verbose)

    # Download each dataset and save it to the directory

    if max_workers:
         with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            executor.map(download_resource, dataset_ids)
    else:
        for dataset_id in dataset_ids: download_resource(dataset_id = dataset_id)
        


# Function to download a file
import requests

def download_file(session, url, headers, file_path, verbose=None, skip_blank=True):
    """Download the corresponding file from the Open Data Portal

    Args:
        session (Session): The session object to use for the request
        url (str): The download URL for the file to try and download it locally
        headers (list): The list of headers to be used in the GET request
        file_path (str): Absolute path of where to download the file
        skip_blank (bool): Whether to skip downloading blank files (default: True)

    Returns:
        int: The length of the downloaded file in bytes
    """
    try:
        response = session.get(url, headers=headers)
        response.raise_for_status()  # Raise an HTTPError for bad responses

        # Decode content based on the response's encoding
        content = response.content.decode(response.encoding or 'utf-8', errors='ignore')
        if content == "NO DATA FOUND" or content.startswith("<html>") or not content.strip():
            if verbose: print(f"Invalid content received from {url}")
            if skip_blank: return 0
        
        # Write the content to the file
        with open(file_path, 'wb') as file:
            file.write(response.content)
        
        return len(response.content)
    except requests.exceptions.RequestException as e:
        print(f"Failed to download {url}: {e}")
        return 0


import unittest
from unittest.mock import patch, MagicMock
import requests
import os
from open_ksa.organizations import organizations

class TestOrganizations(unittest.TestCase):

    @patch('open_ksa.organizations.requests.get')
    def test_organizations_no_parameters(self, mock_get):
        # Mock the response from requests.get
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
    'content': [
        {'id': 'org1', 'name': 'Organization 1', 'publisherID': 'pub1', 'slug': 'org-1', 'numberOfDatasets': 10},
        {'id': 'org2', 'name': 'Organization 2', 'publisherID': 'pub2', 'slug': 'org-2', 'numberOfDatasets': 20},
        {'id': 'org3', 'name': 'Organization 3', 'publisherID': 'pub3', 'slug': 'org-3', 'numberOfDatasets': 30},
        {'id': 'org4', 'name': 'Organization 4', 'publisherID': 'pub4', 'slug': 'org-4', 'numberOfDatasets': 40},
        {'id': 'org5', 'name': 'Organization 5', 'publisherID': 'pub5', 'slug': 'org-5', 'numberOfDatasets': 50},
        {'id': 'org6', 'name': 'Organization 6', 'publisherID': 'pub6', 'slug': 'org-6', 'numberOfDatasets': 60},
        {'id': 'org7', 'name': 'Organization 7', 'publisherID': 'pub7', 'slug': 'org-7', 'numberOfDatasets': 70},
        {'id': 'org8', 'name': 'Organization 8', 'publisherID': 'pub8', 'slug': 'org-8', 'numberOfDatasets': 80},
        {'id': 'org9', 'name': 'Organization 9', 'publisherID': 'pub9', 'slug': 'org-9', 'numberOfDatasets': 90},
        {'id': 'org10', 'name': 'Organization 10', 'publisherID': 'pub10', 'slug': 'org-10', 'numberOfDatasets': 100},
        {'id': 'org11', 'name': 'Organization 11', 'publisherID': 'pub11', 'slug': 'org-11', 'numberOfDatasets': 110}
    ]
    }
        mock_get.return_value = mock_response

        # Call the organizations function without parameters
        result = organizations()

        # Assertions
        mock_get.assert_called_once_with(
            'https://open.data.gov.sa/api/organizations',
            params={'size': 400, 'page': 0, 'sort': 'datasetsCount,DESC'},
            headers={
                'Accept': 'application/json',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': 'https://open.data.gov.sa/',
                'Accept-Language': 'en-US,en;q=0.9',
                'Host': 'open.data.gov.sa',
                'Upgrade-Insecure-Requests': '1'
            }
        )

        # Check if the result contains the expected data
        self.assertEqual(len(result['content']), 11)
        self.assertEqual(result['content'][0]['name'], 'Organization 1')
    @patch('open_ksa.organizations.requests.get')
    def test_organizations_with_parameters(self, mock_get):
        # Mock the response from requests.get
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'content': [
                {'id': 'org1', 'name': 'Organization 1', 'publisherID': 'pub1','slug':'pub-slug', 'numberOfDatasets':100}
            ]
        }
        mock_get.return_value = mock_response

        # Call the organizations function with parameters
        result = organizations(size=1, page=0, sort='datasetsCount,DESC')

        # Assertions
        mock_get.assert_called_once_with(
            'https://open.data.gov.sa/api/organizations',
            params={'size': 1, 'page': 0, 'sort': 'datasetsCount,DESC'},
            headers={
                'Accept': 'application/json',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': 'https://open.data.gov.sa/',
                'Accept-Language': 'en-US,en;q=0.9',
                'Host': 'open.data.gov.sa',
                'Upgrade-Insecure-Requests': '1'
            }
        )

        # Check if the result contains the expected data
        self.assertEqual(len(result['content']), 1)
        self.assertEqual(result['content'][0]['name'], 'Organization 1')

    @patch('open_ksa.organizations.requests.get')
    def test_organizations_json_decode_error(self, mock_get):
        # Mock the response from requests.get
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.side_effect = requests.exceptions.JSONDecodeError("Expecting value", "", 0)
        mock_get.return_value = mock_response

        # Call the organizations function
        result = organizations()

        # Assertions
        mock_get.assert_called_once_with(
            'https://open.data.gov.sa/api/organizations',
            params={'size': 400, 'page': 0, 'sort': 'datasetsCount,DESC'},
            headers={
                'Accept': 'application/json',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
                'Referer': 'https://open.data.gov.sa/',
                'Accept-Language': 'en-US,en;q=0.9',
                'Host': 'open.data.gov.sa',
                'Upgrade-Insecure-Requests': '1'
            }
        )

        # Check if the result is None due to JSON decode error
        self.assertIsNone(result)

if __name__ == '__main__':
    unittest.main()


#examples/scripts/2_get_dataset_resource.py

from open_ksa import get_dataset_resource
def test_get_dataset_resource():
    # Define the dataset ID to test
    dataset_id = 'e63563d0-3312-48f3-8786-7c3e2af61fe7'
    
    # Call the function with the test dataset ID
    get_dataset_resource(dataset_id, verbose=True)

if __name__ == "__main__":
    test_get_dataset_resource()